package buildcraft.silicon;

import java.lang.ref.WeakReference;

import net.minecraft.entity.item.EntityItem;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.inventory.IInventory;
import net.minecraft.inventory.ISidedInventory;
import net.minecraft.inventory.InventoryCrafting;
import net.minecraft.inventory.SlotCrafting;
import net.minecraft.item.ItemStack;
import net.minecraft.item.crafting.IRecipe;
import net.minecraft.nbt.NBTTagCompound;
import net.minecraft.util.EnumFacing;
import net.minecraft.world.WorldServer;

import buildcraft.api.tiles.IHasWork;
import buildcraft.core.lib.gui.ContainerDummy;
import buildcraft.core.lib.inventory.InvUtils;
import buildcraft.core.lib.inventory.StackHelper;
import buildcraft.core.lib.utils.CraftingUtils;
import buildcraft.core.lib.utils.NBTUtils;
import buildcraft.core.lib.utils.BCStringUtils;
import buildcraft.core.lib.utils.Utils;
import buildcraft.core.proxy.CoreProxy;

public class TileStampingTable extends TileLaserTableBase implements IHasWork, ISidedInventory {
    private class LocalInventoryCrafting extends InventoryCrafting {
        public LocalInventoryCrafting() {
            super(new ContainerDummy(), 3, 3);
        }

        private IRecipe findRecipe() {
            return CraftingUtils.findMatchingRecipe(this, field_145850_b);
        }
    }

    private static final int[] SLOTS = Utils.createSlotArray(0, 5);
    private SlotCrafting craftSlot;
    private final LocalInventoryCrafting crafting = new LocalInventoryCrafting();

    public WeakReference<EntityPlayer> getInternalPlayer() {
        return CoreProxy.proxy.getBuildCraftPlayer((WorldServer) field_145850_b, func_174877_v().func_177984_a());
    }

    private void handleLeftoverItems(IInventory items) {
        for (int i = 0; i < items.func_70302_i_(); i++) {
            if (items.func_70301_a(i) != null) {
                ItemStack output = items.func_70301_a(i);

                if (output.field_77994_a <= 0) {
                    items.func_70299_a(i, null);
                    continue;
                }

                boolean inserted = false;

                for (int j = 2; j <= 4; j++) {
                    ItemStack target = func_70301_a(j);

                    if (target == null || target.field_77994_a <= 0) {
                        func_70299_a(j, output);
                        inserted = true;
                        break;
                    } else {
                        output.field_77994_a -= StackHelper.mergeStacks(output, target, true);
                        if (output.field_77994_a == 0) {
                            inserted = true;
                            break;
                        }
                    }
                }

                if (!inserted) {
                    if (output.field_77994_a > 0) {
                        output.field_77994_a -= Utils.addToRandomInventoryAround(field_145850_b, func_174877_v(), output);

                        if (output.field_77994_a > 0) {
                            InvUtils.dropItems(field_145850_b, output, func_174877_v().func_177984_a());
                        }
                    }
                }

                items.func_70299_a(i, null);
            }
        }
    }

    @Override
    public void func_73660_a() {
        super.func_73660_a();

        if (getEnergy() >= getRequiredEnergy() && getEnergy() > 0) {
            ItemStack input = this.func_70301_a(0);

            if (input == null) {
                return;
            }

            EntityPlayer internalPlayer = getInternalPlayer().get();

            if (craftSlot == null) {
                craftSlot = new SlotCrafting(internalPlayer, crafting, this, 1, 0, 0);
            }

            if (input.func_77973_b() instanceof ItemPackage) {
                // Try a recipe made out of the package's contents
                NBTTagCompound tag = NBTUtils.getItemData(input);
                for (int i = 0; i < 9; i++) {
                    if (tag.func_74764_b("item" + i)) {
                        ItemStack is = ItemStack.func_77949_a(tag.func_74775_l("item" + i));
                        if (is != null) {
                            crafting.func_70299_a(i, is);
                        } else {
                            return;
                        }
                    } else {
                        crafting.func_70299_a(i, null);
                    }
                }
            } else {
                // Try a shapeless recipe made from just that item
                ItemStack input2 = input.func_77946_l();
                input2.field_77994_a = 1;
                crafting.func_70299_a(0, input2);
                for (int i = 1; i < 9; i++) {
                    crafting.func_70299_a(i, null);
                }
            }

            IRecipe recipe = crafting.findRecipe();
            ItemStack result = recipe != null ? recipe.func_77572_b(crafting).func_77946_l() : null;

            addEnergy(-getRequiredEnergy());

            if (result != null) {
                craftSlot.func_82870_a(internalPlayer, result);
                handleLeftoverItems(crafting);
                handleLeftoverItems(internalPlayer.field_71071_by);

                for (int i = 1; i <= 4; i++) {
                    ItemStack inside = inv.func_70301_a(i);

                    if (inside == null || inside.field_77994_a <= 0) {
                        inv.func_70299_a(i, result.func_77946_l());
                        result.field_77994_a = 0;
                        break;
                    } else if (StackHelper.canStacksMerge(inside, result)) {
                        result.field_77994_a -= StackHelper.mergeStacks(result, inside, true);

                        if (result.field_77994_a == 0) {
                            break;
                        }
                    }
                }

                if (result.field_77994_a > 0) {
                    EntityItem entityitem = new EntityItem(field_145850_b, func_174877_v().func_177958_n() + 0.5, func_174877_v().func_177956_o() + 0.7, func_174877_v().func_177952_p() + 0.5, result
                            .func_77946_l());

                    field_145850_b.func_72838_d(entityitem);
                    result.field_77994_a = 0;
                }
                func_70298_a(0, 1);
            } else {
                ItemStack outputSlot = func_70301_a(1);
                if (outputSlot == null) {
                    func_70299_a(1, func_70301_a(0));
                    func_70299_a(0, null);
                }
            }
        }
    }

    @Override
    public int getRequiredEnergy() {
        ItemStack stack = this.func_70301_a(0);
        ItemStack output = this.func_70301_a(1);
        if (output != null && output.field_77994_a == output.func_77976_d()) {
            return 0;
        }
        if (stack != null && stack.func_77973_b() != null) {
            if (stack.func_77973_b() instanceof ItemPackage) {
                // tagMap size
                return 400 * NBTUtils.getItemData(stack).func_150296_c().size();
            } else {
                return 400;
            }
        }

        return 0;
    }

    @Override
    public boolean hasWork() {
        return getRequiredEnergy() > 0;
    }

    @Override
    public boolean canCraft() {
        return hasWork();
    }

    @Override
    public int func_70302_i_() {
        return 5;
    }

    @Override
    public String getInventoryName() {
        return BCStringUtils.localize("tile.stampingTableBlock.name");
    }

    @Override
    public boolean func_94041_b(int slot, ItemStack stack) {
        return slot == 0;
    }

    @Override
    public int[] func_180463_a(EnumFacing side) {
        return SLOTS;
    }

    @Override
    public boolean func_180462_a(int slot, ItemStack stack, EnumFacing side) {
        return slot == 0;
    }

    @Override
    public boolean func_180461_b(int slot, ItemStack stack, EnumFacing side) {
        return slot >= 1;
    }
}
